# -*- coding: utf-8 -*-

from django.db import models
import django.contrib
from helpers import add_help_text_headers
from django.contrib.auth.models import User
import datetime

class CustomersAPI(models.Model):
	'''
		3.1 Odbiorcy CUSTOMERS
	'''
	
	user = models.OneToOneField(User)

	key = models.AutoField(primary_key=True, null=True, help_text='symbol')
	nip = models.CharField(max_length=30, null=True, help_text='nip')
	name = models.CharField(max_length=30, null=True, help_text='nazwa')
	person = models.CharField(max_length=30, null=True, help_text='osoba do kontaktu')
	street = models.CharField(max_length=30, null=True, help_text='ulica')
	city = models.CharField(max_length=30, null=True, help_text='miasto')
	zip = models.CharField(max_length=10, null=True, help_text='kod pocztowy')
	email = models.CharField(max_length=255, null=True, help_text='e-mail')
	fax = models.CharField(max_length=30, null=True, help_text='fax')
	tel = models.CharField(max_length=30, null=True, help_text='telefon')
	creditlimit = models.DecimalField(max_digits=11, decimal_places=2, null=True, help_text='limit kredytowy')
	discount = models.DecimalField(max_digits=11, decimal_places=2, null=True, help_text='stały upust 1')
	pricelist = models.DecimalField(max_digits=11, decimal_places=2, null=True, help_text='numer ceny sprzedaży')
	payment = models.CharField(max_length=30, null=True, help_text='sposób płatności')
	maturity = models.IntegerField(null=True, help_text='liczba dni do zapłaty')
	blocked = models.NullBooleanField(null=True, help_text='czy zablokowany')
	vat = models.NullBooleanField(null=True, help_text='czy płatnik VAT')
	note = models.TextField(null=True, help_text='osoba do kontaktu')
	
	@classmethod
	def get(cls, key=None, name=None, email=None):
		if key:
			return cls.objects.get(key=key)
		elif name:
			return cls.objects.get(name=name)
		elif email:
			return cls.objects.get(email=email)
		
		raise User.DoesNotExist
	
	@classmethod
	def authenticate(cls, username, password):
		'''authenticates always (only when using DB, not API)'''
		return cls.objects.get(name=username)

	@classmethod
	def find(cls, key, name, nip):
		customers = CustomersAPI.objects.all()

		if key:
			customers = customers.filter(key__icontains=key)
		if name:
			customers = customers.filter(name__icontains=name)
		if nip:
			customers = customers.filter(nip__icontains=nip)

		return customers


class ProductsAPI(models.Model):
	'''
		3.4 Indeksy magazynowe KIM
	'''
	
	key = models.AutoField(primary_key=True, help_text='indeks')
	code = models.TextField(null=True, help_text='ean13', db_index=True)
	name = models.TextField(null=True, help_text='nazwa', db_index=True)
	PKWiU = models.TextField(help_text='PKWiU')
	packed = models.IntegerField(help_text='ilość/ opak.') # ilość w opakowaniu
	defgroupkey = models.IntegerField(help_text='grupa')
	rebategroupkey = models.IntegerField(null=True, help_text='grupa upustowa')
	defchainkey = models.TextField(help_text='jedn.') # jednostka sprzedaży
	publisher = models.IntegerField(null=True, help_text='dostawca')
	vat = models.IntegerField(null=True, help_text='VAT') # stawka VAT
	price = models.FloatField(null=True, help_text="cena")
	
	def group(self):
		return ProductGroupsAPI.get(self.defgroupkey)

	@classmethod
	@add_help_text_headers
	def get(cls, name, key, code, group, limit, offset):
		rows = cls.objects.all()
		if name:
			rows = rows.filter(name__icontains=name)
		if key:
			rows = rows.filter(key__icontains=key)
		if code:
			rows = rows.filter(code__icontains=code)
		if group:
			group_list = ProductGroupsAPI.list_kids_of(group)
			rows = rows.filter(defgroupkey__in=group_list)
		if offset:
			rows = rows[offset:]
	
		try:
			next = rows[limit]
		except:
			next = None
		
		if limit:
			rows = rows[:limit]

		return rows, next

	
class ProductGroupsAPI(models.Model):
	'''
		3.5 Grupy indeksów
	'''
	
	key = models.AutoField(primary_key=True, help_text='symbol')
	name = models.TextField(help_text='nazwa')
	key_index = models.BigIntegerField(help_text='indeks')
	parent = models.ForeignKey('self', help_text='nadgrupa', null=True)
	
	@classmethod
	def get(cls, key):
		return cls.objects.get(key=key)

	@classmethod
	def kids_of(cls, parent_key):
		if not parent_key:
			rows = cls.objects.filter(parent__isnull=True)
		else:
			parent = cls.objects.get(key=parent_key)
			rows = cls.objects.filter(parent=parent)
		rows = rows.order_by('name')
		return rows

	@classmethod
	def top(cls):
		return cls.kids_of(None)

	@classmethod
	def list_kids_of(cls, parent_key, with_parent=True):
		kids = []
		for kid in cls.kids_of(parent_key):
			kids.append(kid.key)
			if kid.parent:
				kids.extend(cls.list_kids_of(kid.key))
		if with_parent:
			kids.append(parent_key)
		return kids
			
'''
3.6 Dodatkowe opisy z kart KIM STOCKINFO		   
------------------------------------
	 Otrzymamy dodatkowe opisy z kart KIM	
		 - indeks				 PRODUCTKEY 
		 - dodatkowy opis		 INFO 
		 
  3.7 Ceny                   PRICES              
   ---------------------------------
        Otrzymamy aktualnie obowiązujące cenniki w PLN
            - typ                    TYPE       
               katalogowy         SALE          
               dla grup odbiorców GROUP         
               kontraktowy        CUSTOMER      
               promocyjny         PROMO         
            - nazwa                  NAME       
            - nr ceny                IDX        
            - odbiorca,grupa,symbol  EXTID      
            - klucz cennika          KEY        
            - liczyć upust           ALLOWDISCOUNTS
            - rodzaj ceny            PRICETYPE  
               NETTO, BRUTTO 
3.8 Magazyny i stany	   DEPOTS			  
---------------------------------
 Otrzymamy listę magazynów wraz z indeksami i ilościami
 - symbol mag			KEY		 
 - nazwa				 NAME		
 - indeks				PRODUCTKEY  
 - ilość dostępna		VALUE   
 
 	3.9 Stany indeksów		 STOCK			   
--------------------------------
Otrzymamy łączne ilości indeksów w magazynach
 - indeks				PRODUCTKEY  
 - ilość dostępna		VALUE                  
3.11 Upusty od towarów	  DISCOUNTS		   
-------------------------------------
	   Otrzymamy dane o upustach od ceny sprzedaży
		 - symbol				CUSTOMERGROUPKEY
		 - grupa upustowa		PRODUCTGROUPKEY
		 - wartość %			 VALUE  
		 
		                 
3.10 Jednostki miary        CHAINS              
   ----------------------------------
          Otrzymamy tabele jednostek miar
            - symbol                KEY         
            - nazwa                 NAME     
            
'''


class OrdersAPI(models.Model):
	'''
		3.13 Zamówienia sprzedaży ORDERS
	'''
	
	erpkey = models.AutoField(primary_key=True, null=True, help_text='numer')
	user = models.ForeignKey(CustomersAPI, help_text='id clienta')
	customerkey = models.CharField(max_length=30, null=True, help_text='symbol odbiorcy')
	recipent = models.CharField(max_length=30, null=True, help_text='symbol adresata')
	#nr = models.IntegerField(null=True, help_text='numer obcy')
	payment = models.CharField(max_length=30, null=True, help_text='płatność')
	creationdate = models.DateTimeField(null=True, help_text='utworzno')
	paymentdate = models.DateTimeField(null=True, help_text='termin płatności')
	
	@classmethod
	@add_help_text_headers
	def get(cls, user, limit, offset):
		user = User.objects.get(id=user)
		rows = cls.objects.filter(user=CustomersAPI.objects.get(user=user).key).order_by('-erpkey')

		if offset:
			rows = rows[offset:]

		try:
			next = rows[limit]
		except:
			next = None

		if limit:
			rows = rows[:limit]

		return rows, next
	
	@classmethod
	@add_help_text_headers
	def get_by_id(cls, id):
		order = cls.objects.get(erpkey=id) 
		order.positions = PositionsAPI.get(order)
		return order 

	@classmethod
	def create(cls, user, cart, customerkey, payment, paymentdate):
		user = User.objects.get(id=user)
		order = cls.objects.create(user=CustomersAPI.objects.get(user=user))
		order.customerkey = customerkey
		order.payment = payment
		order.creationdate = datetime.datetime.now()
		order.paymentdate = paymentdate
		order.save()
		for p in cart:
			newp = PositionsAPI.objects.create(
				order_id = order,
				customer_id = order.user,
				productkey = p.productkey,
				name = p.name,
				price = p.price,
				quantity = p.quantity,
				chainkey = p.chainkey,
				realizationdate = p.realizationdate
			)
			newp.save()
		return order
	
class PositionsAPI(models.Model):
	idx = models.AutoField(primary_key=True, help_text='numer pozycji')
	order_id = models.ForeignKey(OrdersAPI, db_column='order_id', verbose_name='numer zamówienia')
	customer_id = models.ForeignKey(CustomersAPI, help_text='id odbiorcy')
	productkey = models.IntegerField(help_text='indeks')
	name = models.TextField(help_text='nazwa')
	price = models.FloatField(null=True, help_text='cena')
	quantity = models.FloatField(help_text='ilość')
	realizationdate = models.DateTimeField(null=True, help_text='termin realizacji')
	chainkey = models.CharField(max_length=30, help_text='jedn.')
	
	@classmethod
	@add_help_text_headers
	def get(cls, order):
		return cls.objects.filter(order_id=order)
		
