# -*- coding: utf-8 -*-
# vim: set ai noet sw=4 ts=4 filetype=python fileencoding=utf-8 :

from django.db import models
from django.conf import settings

import urllib2
import httplib

# for version 2.5 of python we have it included...
# from xml.etree.ElementTree import fromstring, tostring, Element

# but we want to support 2.4 + [the same as Djano 1.3 does]
# so we include the library code and install it by itself, and use the 'installed version'
from elementtree.ElementTree import fromstring, tostring, Element

class _str(str):
	def capitalize(s):
		print "capitalize() bypassed: sending value: %s" % ( s )
		return s
		        
	def title(s):
		print "title() bypassed: sending value: %s" % ( s )
		return s


class Client:
	def __init__(self):
		self.target = settings.TRAWERS_API_URL

	def call(self, request):
		try:
			h = urllib2.HTTPHandler(debuglevel=1)
			opener = urllib2.build_opener(h)
			headers = {
				_str('SOAPAction'): request.action,
				_str('Content-type'): 'text/xml',
			}
			http_req = urllib2.Request(self.target, data=request.body, headers=headers)
			http_res = opener.open(http_req)
		except urllib2.HTTPError:
			return None

		response = request.response_cls(http_res.read())
		return response.parse()

class Request:
	def __init__(self, action, body, response_cls):
		self.action = action
		self.body = body
		self.response_cls = response_cls



class Response:
	def __init__(self, body):
		self.body = body
		print body
		self.parse()

	def parse(self):
		self.parse_envelope()
		return self.parse_body()

	def parse_envelope(self):
		self.root = fromstring(self.body)
		if (len(self.root.getchildren()) == 2):
			self.env_header, self.env_body = self.root.getchildren()
		if self.env_body:
			# check for errors
			if (len(self.env_body.getchildren()) == 1):
				self.msg_body = self.env_body.getchildren()[0]
	
	def parse_body(self):
		pass

class ProductsResponse(Response):
	def parse_body(self):
		products = []
		productsCount = None
		if self.msg_body:
			for item in self.msg_body.getchildren():
				if item.tag == 'productsCount': productsCount = int(item.text)
				if item.tag == 'product':
					prod = ProductsAPI()
					for attr in item.getchildren():
						setattr(prod, attr.tag, attr.text)
					products.append(prod)
		return products, productsCount

class ProductResponse(Response):
	def parse_body(self):
		prod = None
		if self.msg_body and self.msg_body.tag == 'product':
			item = self.msg_body
			if item.tag == 'product':
				prod = ProductsAPI()
				for attr in item.getchildren():
					setattr(prod, attr.tag, attr.text)
		return prod

class ProductGroupsResponse(Response):
	def parse_body(self):
		productGroups = []
		productGroupsCount = None
		if self.msg_body:
			for item in self.msg_body.getchildren():
				if item.tag == 'productGroup':
					prod = ProductGroupsAPI()
					for attr in item.getchildren():
						atext = attr.text if attr.text else ''	# coalesce None to ''
						setattr(prod, attr.tag, atext)
					productGroups.append(prod)
		return productGroups, productGroupsCount

class ProductGroupResponse(Response):
	def parse_body(self):
		prod = None
		if self.msg_body and self.msg_body.tag == 'productGroup':
			item = self.msg_body
			if item.tag == 'productGroup':
				prod = ProductGroupsAPI()
				for attr in item.getchildren():
					atext = attr.text if attr.text else ''	# coalesce None to ''
					setattr(prod, attr.tag, atext)
		return prod

class CustomerResponse(Response):
	def parse_body(self):
		cust = None
		if self.msg_body and self.msg_body.tag == 'customer':
			item = self.msg_body
			if item.tag == 'customer':
				cust = CustomersAPI()
				for attr in item.getchildren():
					setattr(cust, attr.tag, attr.text)
		return cust

class OrdersResponse(Response):
	def parse_body(self):
		orders = []
		ordersCount = None
		if self.msg_body:
			for item in self.msg_body.getchildren():
				if item.tag == 'resultsCount': 
					try:
						ordersCount = int(item.text)
					except TypeError:
						ordersCount = 0
				if item.tag == 'erpOrder':
					order = OrdersAPI()
					order.positions = []
					for attr in item.getchildren():
						if attr.tag == 'item':
							position = PositionsAPI()
							order.positions.append(position)
							for pa in attr.getchildren():
								if pa.tag == 'priceInCurrency':
									for pb in pa.getchildren():
										if pb.tag == 'net': position.total_netto = pb.text
										if pb.tag == 'currency': position.total_currency = pb.text
										if pb.tag == 'grossValue': position.total_brutto = pb.text
								else:
									setattr(position, pa.tag, pa.text)
						else:
							setattr(order, attr.tag, attr.text)
					orders.append(order)
		return orders, ordersCount

class OrderResponse(Response):
	def parse_body(self):
		order = None
		if self.msg_body and self.msg_body.tag == 'erpOrder':
			item = self.msg_body
			if item.tag == 'erpOrder':
				order = OrdersAPI()
				order.positions = []
				for attr in item.getchildren():
					if attr.tag == 'item':
						position = PositionsAPI()
						order.positions.append(position)
						for pa in attr.getchildren():
							if pa.tag == 'priceInCurrency':
								for pb in pa.getchildren():
									if pb.tag == 'net': position.total_netto = pb.text
									if pb.tag == 'currency': position.total_currency = pb.text
									if pb.tag == 'grossValue': position.total_brutto = pb.text
							else:
								setattr(position, pa.tag, pa.text)
					else:
						setattr(order, attr.tag, attr.text)
		return order

class OrderResultResponse(Response):
	def parse_body(self):
		order_result = None
		if self.msg_body and self.msg_body.tag == 'orderResult':
			order_result = OrderResultsAPI()
			for attr in self.msg_body.getchildren():
				setattr(order_result, attr.tag, attr.text)
		return order_result

client = Client()

import sys

class AbstractAPI:
	@classmethod
	def build_msg(cls, body_top, session_id='TODO'):
		msg = """<?xml version='1.0' encoding='UTF-8'?>
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/">
   <soapenv:Header>
      <sessionId></sessionId>
   </soapenv:Header>
   <soapenv:Body>
   </soapenv:Body>
</soapenv:Envelope>
"""
		root = fromstring(msg)
		e_session = (root.getchildren()[0]).getchildren()[0]
		e_session.text = session_id

		e_body = (root.getchildren()[1])
		e_body_top = Element(body_top)
		e_body.append(e_body_top)

		return root, e_body_top

	@classmethod
	def tostring(cls, root):
		msg = tostring(root, encoding='utf-8')
		msg = msg.replace('<ns0:', '<soapenv:').replace('</ns0:', '</soapenv:').replace('xmlns:ns0=', 'xmlns:soapenv=')
		return msg


class CustomersAPI:
	key = models.AutoField(primary_key=True, null=True, help_text='symbol')
	nip = models.CharField(max_length=30, null=True, help_text='nip')
	tel = models.CharField(max_length=30, null=True, help_text='telefon')
	vat = models.NullBooleanField(null=True, help_text='czy płatnik VAT')

	zip = models.CharField(max_length=10, null=True, help_text='kod pocztowy')
	city = models.CharField(max_length=30, null=True, help_text='miasto')
	mail = models.CharField(max_length=255, null=True, help_text='e-mail')
	name = models.CharField(max_length=30, null=True, help_text='nazwa')

	note = models.TextField(null=True, help_text='osoba do kontaktu')

	person = models.CharField(max_length=30, null=True, help_text='osoba do kontaktu')
	street = models.CharField(max_length=30, null=True, help_text='ulica')
	enabled = models.NullBooleanField(null=True, help_text='czy dostępny')
	payment = models.CharField(max_length=30, null=True, help_text='sposób płatności')
	discount = models.DecimalField(max_digits=11, decimal_places=2, null=True, help_text='stały upust 1')
	maturity = models.IntegerField(null=True, help_text='liczba dni do zapłaty')
	pricelist = models.DecimalField(max_digits=11, decimal_places=2, null=True, help_text='numer ceny sprzedaży')

	sessionId = models.TextField(null=True)

	creditlimit = models.DecimalField(max_digits=11, decimal_places=2, null=True, help_text='limit kredytowy')
	ordersblocked = models.NullBooleanField(null=True, help_text='czy zablokowany')

	def save(self):
		"""saving to DB disabled"""
		pass

	@classmethod
	def authenticate(cls, name, password, session_id='empty'):
		root, pfind = AbstractAPI.build_msg('CustomerAuthenticate', session_id)
		if name: 
			e = Element('email')
			e.text = "%s" % name
			pfind.append(e)
		if password: 
			e = Element('password')
			e.text = "%s" % password
			pfind.append(e)

		msg = AbstractAPI.tostring(root)
		request = Request('customers', msg, CustomerResponse)
		cust = client.call(request)
		
		return cust

	# XXX - server does not implement this call
	@classmethod
	def get(cls, key=None, name=None, email=None, session_id='empty'):
		if key: 
			root, pfind = AbstractAPI.build_msg('customerGetByKey', session_id)
			e = Element('key')
			e.text = "%s" % key
			pfind.append(e)
		if name: 
			root, pfind = AbstractAPI.build_msg('customerGetByName', session_id)
			e = Element('name')
			e.text = "%s" % name
			pfind.append(e)
		if email:
			root, pfind = AbstractAPI.build_msg('customerGetByEmail', session_id)
			e = Element('email')
			e.text = "%s" % email
			pfind.append(e)

		if root and pfind:
			msg = AbstractAPI.tostring(root)
			request = Request('customers', msg, Response)
			rows, rows_count = client.call(request)
		
			return rows, next
		
		raise User.DoesNotExist
	
	# XXX - server does not implement this call
	@classmethod
	def find(cls, key, name, nip):
		root, pfind = AbstractAPI.build_msg('customersFind', session_id)
		if name: 
			e = Element('name')
			e.text = "%s" % name
			pfind.append(e)
		if key: 
			e = Element('key')
			e.text = "%s" % key
			pfind.append(e)
		if nip:
			e = Element('nip')
			e.text = "%s" % nip
			pfind.append(e)

		msg = AbstractAPI.tostring(root)
		request = Request('customers', msg, Response)
		rows, rows_count = client.call(request)
		
		return rows, next

class ProductsAPI:
	key = models.AutoField(primary_key=True, help_text='indeks')
	vat = models.IntegerField(null=True, help_text='VAT') # stawka VAT
	code = models.TextField(null=True, help_text='ean13', db_index=True)
	name = models.TextField(null=True, help_text='nazwa', db_index=True)
	PKWiU = models.TextField(help_text='PKWiU')
	#price = models.FloatField(null=True, help_text="cena", default=None)
	price = None
	packed = models.IntegerField(help_text='ilość/ opak.') # ilość w opakowaniu
	currency = models.TextField(help_text='waluta')
	publisher = models.IntegerField(null=True, help_text='dostawca')
	defChainKey = models.TextField(help_text='jedn.') # jednostka sprzedaży
	defGroupKey = models.TextField(help_text='grupa')
	rebateGroupKey = models.IntegerField(null=True, help_text='grupa upustowa')
	
	def group(self):
		#return ProductGroupsAPI.get(self.defgroupkey)
		return self.defGroupKey
	
	def set_full_group(self, group):
		self._full_group = group
	
	def has_full_group(self):
		try:
			return self._full_group is not None
		except AttributeError:
			return False
		return False
	
	def full_group(self, session_id='empty'):
		if not self.has_full_group():
			self._full_group = ProductGroupsAPI.get(self.defGroupKey, session_id=session_id)
		return self._full_group
	
	@classmethod
	def get(cls, name, key, code, group, limit, offset, do_cache_groups=True, cached_groups={}, session_id='empty'):
		root, pfind = AbstractAPI.build_msg('productsFind', session_id)
		if name: 
			e = Element('name')
			e.text = "%s" % name
			pfind.append(e)
		if key: 
			e = Element('key')
			e.text = "%s" % key
			pfind.append(e)
		if code:
			e = Element('code')
			e.text = "%s" % code
			pfind.append(e)
		# XXX - server side - not implemented
		if group:
			e = Element('defgroupKey')
			e.text = "%s" % group
			pfind.append(e)
		
		if not limit: limit = 10
		if limit > 10: limit = 15
		if limit and not offset: offset = 0

		if limit:
			e = Element('limit')
			e.text = "%s" % limit
			pfind.append(e)
		if offset:
			e = Element('offset')
			e.text = "%s" % offset
			pfind.append(e)
		
		next = None
		
		msg = AbstractAPI.tostring(root)
		request = Request('products', msg, ProductsResponse)
		rows, rows_count = client.call(request)
		

		# get full group data for all products
		# use cache of groups, to reduce nr of requests - every group details is one request

		for prod in rows:
			# product does not have a full group
			if do_cache_groups and not prod.has_full_group():
				# if not in cache, then fetch and store in cache
				if not prod.group() in cached_groups:
					full_group = prod.full_group(session_id=session_id)
					cached_groups[prod.group()] = full_group
				# if group in cache then set to prod
				else:
					prod.set_full_group(cached_groups[prod.group()])
		
		if rows_count and (rows_count > (offset + limit)): next = True
		
		return rows, next

	@classmethod
	def get_by_key(cls, key, session_id='empty'):
		root, find_by_key = AbstractAPI.build_msg('productGetByKey', session_id)
		if key: 
			e = Element('key')
			e.text = "%s" % key
			find_by_key.append(e)

		msg = AbstractAPI.tostring(root)
		request = Request('products', msg, ProductResponse)
		product = client.call(request)

		return product

class ProductGroupsAPI:
	key = models.TextField(help_text='symbol')
	name = models.TextField(help_text='nazwa')
	parent = models.TextField(help_text='parent defgroupKey')

	def __init__(self):
		self.parent = ''
	
	@classmethod
	def get(cls, key, session_id='empty'):
		root, find_by_key = AbstractAPI.build_msg('productGroupGetByKey', session_id)
		if key: 
			e = Element('defgroupKey')
			e.text = "%s" % key
			find_by_key.append(e)

		msg = AbstractAPI.tostring(root)
		request = Request('productGroups', msg, ProductGroupResponse)
		order = client.call(request)

		return order

	@classmethod
	def kids_of(cls, parent, session_id='empty'):
		root, find_by_key = AbstractAPI.build_msg('productGroupsSubGroupsOf', session_id)

		if parent:
			e = Element('defgroupKey')
			e.text = "%s" % parent
			find_by_key.append(e)

		msg = AbstractAPI.tostring(root)
		request = Request('productGroups', msg, ProductGroupsResponse)
		rows, rows_count = client.call(request)
		return rows

	@classmethod
	def top(cls, session_id='empty'):
		return cls.kids_of('', session_id=session_id)

	@classmethod
	def list_kids_of(cls, parent, with_parent=True, session_id='empty'):
		kids = []
		for kid in cls.kids_of(parent, session_id=session_id):
			kids.append(kid.key)
			if kid.parent:
				kids.extend(cls.list_kids_of(kid.key, session_id=session_id))
		if with_parent:
			kids.append(parent if parent else '')
		return kids
			
class OrdersAPI:
	erpKey = models.TextField(null=True, help_text='numer')
	nr = models.IntegerField(null=True, help_text='numer obcy')
	repKey = models.CharField(max_length=30, null=True, help_text='repkey')
	type = models.CharField(max_length=30, null=True, help_text='typ')
	payment = models.CharField(max_length=30, null=True, help_text='płatność')
	customerKey = models.CharField(max_length=30, null=True, help_text='symbol odbiorcy')
	paymentDate = models.DateTimeField(null=True, help_text='termin płatności')
	creationDate = models.DateTimeField(null=True, help_text='utworzono')
	realizationDate = models.DateTimeField(null=True, help_text='termin realizacji')
	description = ''

	@classmethod
	def get(cls, user, limit, offset, session_id='empty'):
		root, pfind = AbstractAPI.build_msg('ordersGet', session_id)
		if not limit: limit = 10
		if limit > 10: limit = 15
		if limit and not offset: offset = 0

		if limit:
			e = Element('limit')
			e.text = "%s" % limit
			pfind.append(e)
		if offset:
			e = Element('offset')
			e.text = "%s" % offset
			pfind.append(e)

		next = None

		msg = AbstractAPI.tostring(root)
		request = Request('orders', msg, OrdersResponse)
		rows, rows_count = client.call(request)
		
		if rows_count and (rows_count > (offset + limit)): next = True
		
		return rows, next


	@classmethod
	def get_by_id(cls, id, session_id='empty'):
		root, find_by_id = AbstractAPI.build_msg('orderGetByID', session_id)
		if id: 
			e = Element('orderid')
			e.text = "%s" % id
			find_by_id.append(e)

		msg = AbstractAPI.tostring(root)
		request = Request('orders', msg, OrderResponse)
		order = client.call(request)

		return order

	@classmethod
	def create(cls, cart_items, description, session_id='empty'):
		root, onew = AbstractAPI.build_msg('orderNew', session_id)
		for item in cart_items:
			it = Element('item')
			
			pk = Element('productKey')
			pk.text = "%s" % item.productkey
			it.append(pk)
			
			qt = Element('quantity')
			qt.text = "%s" % item.quant
			it.append(qt)
			
			onew.append(it)

		dsc = Element('description')
		dsc.text = "%s" % description
		onew.append(dsc)
			
		order = OrdersAPI()
		order.msg = AbstractAPI.tostring(root)
		print order.msg
		return order
	
	def save(self):
		request = Request('orders', self.msg, OrderResultResponse)
		order_result = client.call(request)
		self.order_result = order_result
		return order_result
	
	def calc_totals(self):
		netto = 0.0
		brutto = 0.0
		currency = None
		for position in self.positions:
			netto += float(position.total_netto)
			brutto += float(position.total_brutto)
			if not currency and position.total_currency:
				currency = position.total_currency
		self.total_netto = netto
		self.total_brutto = brutto
		self.total_currency = currency	

class OrderResultsAPI:
	erpKey = models.TextField(null=True)
	result = None
	adeptKey = None
	faultstring = None
	
	def error_message(self):
		msg = u'Wystąpił błąd podczas dodawania zlecenia.'
		if self.faultstring:
			msg += unicode(self.faultstring)
		return msg
	
	def is_success(self):
		b1 = self.result
		b2 = self.erpKey
		b3 = unicode(self.result) != ''
		b4 = unicode(self.erpKey) != ''
		b5 = 'ERR'.lower() not in (unicode(self.result)).lower()
		return b1 and b2 and b3 and b4 and b5

class PositionsAPI(models.Model):
	idx = models.AutoField(primary_key=True, help_text='numer pozycji')
	price = models.FloatField(null=True, help_text='cena')
	chainKey = models.CharField(max_length=30, help_text='jedn.')
	discount = models.FloatField(help_text='rabat')
	quantity = models.FloatField(help_text='ilość')
	productKey = models.TextField(help_text='indeks')
	productName = models.TextField(help_text='nazwa')
	total_netto = models.FloatField(help_text='Wartość netto')
	total_brutto = models.FloatField(help_text='Wartość brutto')
	total_currency = models.TextField(help_text='waluta')

